@@ -1844,7 +1844,11 @@ def test_async_s3(s3):
         # with s3.open(fn, "rb") as f:
         #     assert f.read() == data
 
-        await session.close()
+        try:
+            await session.close()
+        except AttributeError:
+            # bug in aiobotocore 1.4.1
+            await session._endpoint.http_session._session.close()
 
     asyncio.run(_())
 
@@ -1877,7 +1881,11 @@ def test_async_s3_old(s3):
         # Check old API
         session = await s3._connect()
         assert await s3._cat_file(fn, start=0, end=3) == data[:3]
-        await session.close()
+        try:
+            await session.close()
+        except AttributeError:
+            # bug in aiobotocore 1.4.1
+            await session._endpoint.http_session._session.close()
 
     asyncio.run(_())
 
@@ -1934,12 +1942,26 @@ def test_async_close():
                     future.result()
 
         await asyncio.gather(*[async_wrapper() for __ in range(2)])
-
-        await s3._s3.close()
+        try:
+            await s3._s3.close()
+        except AttributeError:
+            # bug in aiobotocore 1.4.1
+            await s3._s3._endpoint.http_session._session.close()
 
     asyncio.run(_())
 
 
+def test_put_single(s3, tmpdir):
+    fn = os.path.join(str(tmpdir), "dir")
+    os.mkdir(fn)
+    open(os.path.join(fn, "abc"), "w").write("text")
+    s3.put(fn + "/", test_bucket_name)  # no-op, no files
+    assert not s3.exists(test_bucket_name + "/abc")
+    assert not s3.exists(test_bucket_name + "/dir")
+    s3.put(fn + "/", test_bucket_name, recursive=True)  # no-op, no files
+    assert s3.cat(test_bucket_name + "/dir/abc") == b"text"
+
+
 def test_shallow_find(s3):
     """Test that find method respects maxdepth.
 
@@ -2226,3 +2248,9 @@ def test_rm_file(s3):
     s3.rm_file(target)
     assert not s3.exists(target)
     assert not s3.exists(test_bucket_name + "/to_be_removed")
+
+
+def test_exists_isdir(s3):
+    bad_path = "s3://nyc-tlc-asdfasdf/trip data/"
+    assert not s3.exists(bad_path)
+    assert not s3.isdir(bad_path)
